package cz.fidentis.analyst.tests;

import cz.fidentis.analyst.face.HumanFace;
import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.visitors.mesh.Curvature;
import cz.fidentis.analyst.visitors.mesh.DeltaCurvature;
import cz.fidentis.analyst.visitors.mesh.GaussianCurvature;
import cz.fidentis.analyst.visitors.mesh.MeanCurvature;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * A class for testing curvature algorithms.
 * 
 * @author radek oslejsek
 */
public class CurvatureTests {

    private static File faceFile2 = new File("src/test/resources/cz/fidentis/analyst/00002_01_ECA.obj");
    
    /**
     * Main method 
     * @param args Input arguments 
     * @throws IOException on IO error
     */
    public static void main(String[] args) throws IOException {
        HumanFace face = new HumanFace(faceFile2);
        
        boolean print = false;
        
        System.out.println();
        System.out.println("Curvature calculation:");
        System.out.println(measureCurvature(face, new GaussianCurvature(), print) + "\tmsec:\tGaussian curvature");
        System.out.println(measureCurvature(face, new MeanCurvature(), print) + "\tmsec:\tMean curvature");
        System.out.println(measureCurvature(face, new DeltaCurvature(true), print) + "\tmsec:\tMax curvature");
        System.out.println(measureCurvature(face, new DeltaCurvature(false), print) + "\tmsec:\tMin curvature");
        System.out.println();
    }
    
    private static long measureCurvature(HumanFace face, Curvature vis, boolean printDetails) {
        long startTime = System.currentTimeMillis();
        face.getMeshModel().compute(vis);
        long retTime =  System.currentTimeMillis() - startTime;
        
        if (printDetails) {
            Map<MeshFacet, List<Double>> results = vis.getCurvatures();
            for (MeshFacet facet: results.keySet()) {
                System.out.println(results.get(facet));
            }
        }
        
        return retTime;
    }
}
