package cz.fidentis.analyst.visitors.mesh;

import cz.fidentis.analyst.mesh.core.MeshFacet;
import java.util.List;

/**
 * Gaussian curvatures. 
 * @see https://computergraphics.stackexchange.com/questions/1718/what-is-the-simplest-way-to-compute-principal-curvature-for-a-mesh-triangle
 * 
 * @author Natalia Bebjakova
 * @author Radek Oslejsek
 */
public class GaussianCurvature extends Curvature {
    
    @Override
    protected double calculateCurvature(MeshFacet facet, List<CurvTriangle> triangles, int vertA) {
        List<Integer> neighbouringTriangles = facet.getCornerTable().getTriangleIndexesByVertexIndex(vertA);
        
        if (neighbouringTriangles.isEmpty()) {
            return Double.NaN;
        }
        
        double sumArea = 0.0;
        double sumAngles = 0.0;
        
        // for all surrounding triangles:
        for (int i = 0; i < neighbouringTriangles.size(); i++) {
            CurvTriangle ctri = triangles.get(neighbouringTriangles.get(i));
            sumAngles +=  ctri.alpha(facet.getVertex(vertA));
            sumArea += computeArea(ctri, facet.getVertex(vertA));
        }
        
        return (2.0 * Math.PI - sumAngles) / sumArea;        
    }
    
    
    /*
    @Override
    protected double calculateCurvature(MeshFacet facet, List<MeshTriangle> triangles, int centerIndex) {
        List<Integer> neighbouringTriangles = facet.getCornerTable().getTriangleIndexesByVertexIndex(centerIndex);
        
        if (neighbouringTriangles.isEmpty()) {
            return Double.NaN;
        }
        
        double sum = 2 * Math.PI;
        double areaSum = 0;

        // fro all surrounding triangles:
        List<Vector3d> voronoiPoints = facet.calculateVoronoiPoints();
        for (int i = 0; i < neighbouringTriangles.size(); i++) {
            Vector3d v1 = new Vector3d();
            Vector3d v2 = new Vector3d();
            MeshTriangle tri = triangles.get(neighbouringTriangles.get(i));
            Vector3d voronoiPoint = voronoiPoints.get(neighbouringTriangles.get(i));
            
            double area = 0;
            if (tri.vertex1 ==  facet.getVertex(centerIndex)) {
                v1 = new Vector3d(tri.vertex2.getPosition());
                v2 = new Vector3d(tri.vertex3.getPosition());
                v1.sub(tri.vertex1.getPosition());
                v2.sub(tri.vertex1.getPosition());
                area = voronoiPoint.x;
            } else if (tri.vertex2 == facet.getVertex(centerIndex)) {
                v1 = new Vector3d(tri.vertex1.getPosition());
                v2 = new Vector3d(tri.vertex3.getPosition());
                v1.sub(tri.vertex2.getPosition());
                v2.sub(tri.vertex2.getPosition());
                area = voronoiPoint.y;
            } else if (tri.vertex3 == facet.getVertex(centerIndex)) {
                v1 = new Vector3d(tri.vertex2.getPosition());
                v2 = new Vector3d(tri.vertex1.getPosition());
                v1.sub(tri.vertex3.getPosition());
                v2.sub(tri.vertex3.getPosition());
                area = voronoiPoint.z;
            }
            
            areaSum += area;
            v1.normalize();
            v2.normalize();
            
            sum -= Math.acos(v1.dot(v2));
        } 
        double value = sum * (1.0 / areaSum);
        
        return value;        
    }
    */
}
